package xyz.veronie.bgg.result;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.LinkOption;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;

import xyz.veronie.bgg.ui.helpers.Resources;

public class Thing {

	private int id;
	private ThingMetaData metaData;
	private ThingDetails details;
	
	// cache
	private Image thumbImage;
		
	public static final String IdHeader = "ID";
	public static final String ThumbHeader = "Thumbnail";
	public static final String NameHeader = "Name";
	public static final String DetailsHeader = "Details";
	public static final String ExportFlagHeader = "Export";
	
	public Thing(int id, ThingMetaData metaData) {
		this.id = id;
		this.metaData = metaData;
	}
	
	
	public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}
	public ThingMetaData getMetaData() {
		return metaData;
	}
	public void setMetaData(ThingMetaData metaData) {
		this.metaData = metaData;
	}
	public ThingDetails getDetails() {
		return details;
	}
	public void setDetails(ThingDetails details) {
		this.details = details;
	}
	
	public Image getThumbnail() {
		if(thumbImage == null) {
			getThumbImage(id);
		}
		return thumbImage;
	}
	
	private void getThumbImage(int thingId) {
		Path tmpDir = Resources.INSTANCE.getTmpDir();
		if(tmpDir != null) {
			Path imageFile = Paths.get(Resources.INSTANCE.getThumbsDir().toString()
					+ File.separator + String.valueOf(thingId) + ".png");
			
			if(Files.exists(imageFile, LinkOption.NOFOLLOW_LINKS)) {
				String filename = imageFile.toString().replace("\\","/");
				thumbImage = Resources.INSTANCE.getResourceManager().createImage(
						ImageDescriptor.createFromFile(null, filename));				
			} else {
				try {
					ImageDescriptor descriptor = ImageDescriptor.createFromURL(
							new URL(metaData.getThumbURL()));
					thumbImage = descriptor.createImage();

					ImageLoader saver = new ImageLoader();
					saver.data = new ImageData[] { thumbImage.getImageData() };
					saver.save(imageFile.toString(), SWT.IMAGE_PNG);					
				} catch (MalformedURLException e) {
					System.out.println("INFO: Malformed URL for ThingId " + thingId);
				}
			}

		}
	}


	/// get field at idx, order of titles
    public String getField(int idx) {
    	
    	String returnStr;

    	switch(idx) {
    	case 0:
    		returnStr = "";
    		break;
    	case 1:
    		returnStr = this.getMetaData().getName();
    		break;
    	case 2:
    		returnStr = String.valueOf(this.id);
    		break;
    	case 3:
    		returnStr = (this.getDetails() != null) ? "yes" : "no";
    		break;
    	case 4:
    		returnStr = this.getMetaData().getName();
    		break;
    	default:
    		throw new ArrayIndexOutOfBoundsException(
    				"idx " + String.valueOf(idx) + " must be in [0,4]");
    	}

    	return returnStr;
    }
    
    
    @Override
    public boolean equals(Object o) {
    	if(o instanceof Thing) {
    		return ((Thing)o).getId() == getId();
    	} else {
    		return false;
    	}
    }
	
}
