package xyz.veronie.bgg.ui.filters.composites;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import xyz.veronie.bgg.result.ResultConfigManager;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.types.Subtype;
import xyz.veronie.bgg.types.UserFlag;

public class BggUserSourceComposite extends Composite {

	private IEventBroker eventBroker;
	 
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public BggUserSourceComposite(Composite parent, int style, 
			IEventBroker eventBroker, ResultConfigManager configManager) 
	{
		super(parent, style);
		this.eventBroker = eventBroker;		
		
		Composite main = new Composite(parent, SWT.FILL);
		main.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		main.setLayout(new GridLayout(1, false));
		
		Composite topRowComposite = new Composite(main, SWT.NONE);
		topRowComposite.setLayout(new GridLayout(5, false));
		topRowComposite.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, true, false, 1, 1));
		
		Label lblUserName = new Label(topRowComposite, SWT.NONE);
		lblUserName.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblUserName.setText("User name:");
		
		ComboViewer comboViewerUserName = new ComboViewer(topRowComposite, SWT.NONE);
		Combo cbUserName = comboViewerUserName.getCombo();
		cbUserName.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent e) {
				setUser(cbUserName.getText());
			}
		});
		GridData gd_cbUserName = new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1);
		gd_cbUserName.widthHint = 100;
		cbUserName.setLayoutData(gd_cbUserName);
        cbUserName.setText(configManager.getResultConfig().user);

		
		new Label(topRowComposite, SWT.NONE);
		
		Label lblSubtype = new Label(topRowComposite, SWT.NONE);
		lblSubtype.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblSubtype.setText("Subtypes:");
		
		ComboViewer comboViewerSubtypes = new ComboViewer(topRowComposite, SWT.NONE);
		Combo cbSubtypes = comboViewerSubtypes.getCombo();
		cbSubtypes.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		comboViewerSubtypes.setContentProvider(new ArrayContentProvider());
		comboViewerSubtypes.setInput(getSubtypes());
		comboViewerSubtypes.setSelection(new StructuredSelection(configManager.getResultConfig().subtype));
		comboViewerSubtypes.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				eventBroker.send(EventConstants.TOPIC_SUBTYPE_CHANGED, selection.getFirstElement());
			}
		});

		
		Composite centerComposite = new Composite(main, SWT.NONE);
		centerComposite.setLayout(new GridLayout(1, false));
		centerComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
//		centerComposite.setLayout(new FillLayout(SWT.VERTICAL));
		
		Label infoLabel = new Label(centerComposite, SWT.LEFT);
        infoLabel.setText("Select which flags are used as filter. Filters follow 'AND' rule.");
        GridData gdInfo = new GridData(SWT.FILL, SWT.FILL, true, false);
        infoLabel.setLayoutData(gdInfo);

        new FilterFlagComposite(centerComposite, UserFlag.OWN, configManager, eventBroker);
        new FilterFlagComposite(centerComposite, UserFlag.PREORDERED, configManager, eventBroker);
        new FilterFlagComposite(centerComposite, UserFlag.WISHLIST, configManager, eventBroker);
        new FilterFlagComposite(centerComposite, UserFlag.WTB, configManager, eventBroker);
        new FilterFlagComposite(centerComposite, UserFlag.WTP, configManager, eventBroker);
        new FilterFlagComposite(centerComposite, UserFlag.WANT, configManager, eventBroker);
        new FilterFlagComposite(centerComposite, UserFlag.PREVIOUSLY_OWNED, configManager, eventBroker);
        new FilterFlagComposite(centerComposite, UserFlag.PLAYED, configManager, eventBroker);		
        new FilterFlagComposite(centerComposite, UserFlag.TRADE, configManager, eventBroker);        
        new FilterFlagComposite(centerComposite, UserFlag.RATED, configManager, eventBroker);
        new FilterFlagComposite(centerComposite, UserFlag.COMMENT, configManager, eventBroker);
        
        this.pack();
	}


	private void setUser(String user) {
		if(eventBroker != null) {
			eventBroker.post(EventConstants.TOPIC_USER_CHANGED, user);
		} else {
			System.out.println("setGeeklistId: eventBroker is null.");
		}				
	}
	
	public List<Subtype> getSubtypes() {
		List<Subtype> subtypes = new ArrayList<Subtype>();
		for (Subtype st : Subtype.values()) { 
			subtypes.add(st);
		}
		return subtypes;
	}

	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
