 
package xyz.veronie.bgg.ui.parts;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.EPartService.PartState;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.wb.swt.ResourceManager;
import org.eclipse.wb.swt.SWTResourceManager;

import xyz.veronie.bgg.result.BggApi;
import xyz.veronie.bgg.result.ResultConfig;
import xyz.veronie.bgg.result.ResultConfigManager;
import xyz.veronie.bgg.result.Thing;
import xyz.veronie.bgg.result.ThingProvider;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.types.ResultAction;
import xyz.veronie.bgg.types.SourceFilter;
import xyz.veronie.bgg.ui.filters.BggUserSourceFilter;
import xyz.veronie.bgg.ui.filters.FamilySourceFilter;
import xyz.veronie.bgg.ui.filters.GeeklistSourceFilter;
import xyz.veronie.bgg.ui.helpers.BatColors;
import xyz.veronie.bgg.ui.helpers.BatLayouts;

public class FetchPart {
	
	@Inject
	private ResultConfigManager configManager;
	
	// inject all source filter composites
	@Inject private BggUserSourceFilter bggUserSourceFilter;
	@Inject private GeeklistSourceFilter geeklistSourceFilter;
	@Inject private FamilySourceFilter familySourceFilter;

	@Inject 
	private IEventBroker eventBroker;
	
	@Inject 
	private MApplication application;
	
	@Inject
	private EModelService modelService;
	
	@Inject 
	private EPartService partService;
	
	@Inject 
	private ThingProvider thingProvider;
	
	@Inject 
	private BggApi bggApi;
	
	private Button btnBggUser;
	private Button btnFamily;
	private Button btnGeeklist;

	private Composite main;
	private Composite filterComposite;

	
	@Inject
	public FetchPart() {
		
	}
	
	@PostConstruct
	public void postConstruct(Composite parent) {
		parent.setBackground(BatColors.getBackgroundColor());
		parent.setBackgroundMode(SWT.INHERIT_FORCE);
		parent.setLayout(new GridLayout(1, false));
		
		main = new Composite(parent, SWT.NONE);
		GridLayout gl_main = new GridLayout(1, false);
		BatLayouts.applyStandardSpacing(gl_main);
		main.setLayout(gl_main);
		main.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, true, true, 1, 1));
		
		Composite buttonRow = new Composite(main, SWT.NONE);
		buttonRow.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
		GridLayout gl_buttonRow = new GridLayout(3, false);
		BatLayouts.applyStandardSpacing(gl_buttonRow);
		buttonRow.setLayout(gl_buttonRow);
		
		btnBggUser = new Button(buttonRow, SWT.NONE);		
		btnBggUser.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/noun_Meeple_60x60.png"));
		
		btnFamily = new Button(buttonRow, SWT.NONE);
		btnFamily.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/noun_Family_60x60.png"));
		
		btnGeeklist = new Button(buttonRow, SWT.NONE);
		btnGeeklist.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/noun_List_60x60.png"));
		
		
		btnBggUser.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				selectFilter(SourceFilter.BGG_USER);
			}
		});

		btnFamily.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				selectFilter(SourceFilter.FAMILY);
			}
		});
		
		btnGeeklist.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				selectFilter(SourceFilter.GEEKLIST);
			}
		});
		

		Composite centerComposite = new Composite(main, SWT.NONE);
		centerComposite.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, true, true, 1, 1));
		GridLayout gl_centerComposite = new GridLayout(1, false);
		BatLayouts.applyStandardSpacing(gl_centerComposite);
		centerComposite.setLayout(gl_centerComposite);
		
		
		filterComposite = new Composite(centerComposite, SWT.NONE);
		filterComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
		filterComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		filterComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_filterComposite = new GridLayout(1, false);
		BatLayouts.applyStandardSpacing(gl_filterComposite);
		filterComposite.setLayout(gl_filterComposite);

		
		
		Composite applyComposite = new Composite(main, SWT.NONE);
		GridLayout gl_applyComposite = new GridLayout(5, false);
		BatLayouts.applyStandardSpacing(gl_applyComposite);
		applyComposite.setLayout(gl_applyComposite);
		applyComposite.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, true, false, 1, 1));
		
		Button btnReplace = new Button(applyComposite, SWT.NONE);
		btnReplace.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/result_replace_60x60.png"));
		btnReplace.setToolTipText("Replace");
		btnReplace.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				fetchEntries(ResultAction.REPLACE);
			}
		});
		
		Button btnOnlyNew = new Button(applyComposite, SWT.NONE);
		btnOnlyNew.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/only_new_60x60.png"));
		btnOnlyNew.setToolTipText("Keep only new");
		btnOnlyNew.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				fetchEntries(ResultAction.ONLY_NEW);
			}
		});
		
		Button btnAdd = new Button(applyComposite, SWT.NONE);
		btnAdd.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/result_add_60x60.png"));
		btnAdd.setToolTipText("Add");
		btnAdd.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				fetchEntries(ResultAction.ADD);
			}
		});
		
		Button btnIntersect = new Button(applyComposite, SWT.NONE);
		btnIntersect.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/result_intersect_60x60.png"));
		btnIntersect.setToolTipText("Intersect");
		btnIntersect.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				fetchEntries(ResultAction.INTERSECT);
			}
		});
		
		Button btnSubtract = new Button(applyComposite, SWT.NONE);
		btnSubtract.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/result_subtract_60x60.png"));
		btnSubtract.setToolTipText("Subtract");
		btnSubtract.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				fetchEntries(ResultAction.SUBTRACT);
			}
		});
		
		// init filter using config manager
		SourceFilter source = configManager.getResultConfig().source;
		selectFilter(source);
		showFilter(filterComposite, source); 


	}
	
	
	private void selectFilter(SourceFilter source) {
		configManager.getResultConfig().source = source;

		switch(source) {
		default:
		case BGG_USER:
			btnBggUser.setFocus();
			btnBggUser.setBackground(BatColors.getButtonBgColor());
			btnFamily.setBackground(null);
			btnGeeklist.setBackground(null);
			break;
		case FAMILY:
			btnFamily.setFocus();
			btnBggUser.setBackground(null);
			btnFamily.setBackground(BatColors.getButtonBgColor());
			btnGeeklist.setBackground(null);
			break;
		case GEEKLIST:
			btnGeeklist.setFocus();
			btnBggUser.setBackground(null);
			btnFamily.setBackground(null);
			btnGeeklist.setBackground(BatColors.getButtonBgColor());
			break;
		}
		showFilter(filterComposite, source);
		eventBroker.send(EventConstants.TOPIC_SOURCE_CHANGED, source);
	}
	
	
	/// show different filter controls depending on selection in cbSource ComboViewer
	private void showFilter(Composite parent, SourceFilter source) {	

		// clean up
		for(Control child : parent.getChildren()) {
			child.dispose();
		}
		
		// create a new filter area based on selection:
		switch(source) {
		default:
		case BGG_USER:
			System.out.println("construct " + source);
			bggUserSourceFilter.create(parent, SWT.FILL);
			break;
		case GEEKLIST:
			System.out.println("construct " + source);
			geeklistSourceFilter.create(parent, SWT.FILL);
			break;
		case FAMILY:
			System.out.println("construct " + source);
			familySourceFilter.create(parent, SWT.FILL);
			break;
		}

		main.pack();
		main.layout(true, true);
	}
	
	
	private void fetchEntries(ResultAction action) {
        Object fetchId = getFetchId();        
        if(fetchId == null) return;
        
		try {
			SourceFilter source = configManager.getResultConfig().source;
	        eventBroker.send(EventConstants.TOPIC_STATUS, "Fetching " + source.toString() + " '" + fetchId + "'...");
			ArrayList<Thing> things = bggApi.getThings(source, fetchId);
        	useThingsOnResult(things, action);
			goToHomePart();
		}
        catch(IllegalArgumentException ex) {
        	MessageDialog.openError(main.getShell(), "", ex.getMessage());
        }
    }

	private Object getFetchId() {
        ResultConfig resultConfig = configManager.getResultConfig();
        SourceFilter source = configManager.getResultConfig().source;
        
		if(source == SourceFilter.BGG_USER) {
        	String user = resultConfig.user;
        	if(user == null || user.isEmpty()) {
        		MessageDialog.openError(main.getShell(), "", "Please enter a user name.");
        		return null;
        	} 
        	return user;
        	
        } else if(source == SourceFilter.GEEKLIST) {
        	Integer geeklistId = resultConfig.geeklistId;
        	if(geeklistId == null) {
        		MessageDialog.openError(main.getShell(), "", "Please enter a geeklist id.");
        		return null;
        	} 
        	return geeklistId;
        } else if(source == SourceFilter.FAMILY) {
        	Integer familyId = resultConfig.familyId;
        	if(familyId == null) {
        		MessageDialog.openError(main.getShell(), "", "Please enter a family id.");
        		return null;
        	}
        	return familyId;
        }

		return null;
	}

	private void useThingsOnResult(ArrayList<Thing> things, ResultAction action) {
		switch(action) {
		case REPLACE:
			thingProvider.replaceThings(things);
			break;
		case ADD:
			thingProvider.addThings(things);
			break;
		case SUBTRACT:
			thingProvider.subtractThings(things);
			break;
		case INTERSECT:
			thingProvider.intersectThings(things);
			break;
		case ONLY_NEW:
			thingProvider.keepOnlyNew(things);
			break;
		}
		eventBroker.send(EventConstants.TOPIC_RESULT_CHANGED, "");
		eventBroker.send(EventConstants.TOPIC_STATUS, "Fetched " + Integer.toString(things.size()) + " things.");
	}
	
	
	private void goToHomePart() {
		// TODO Auto-generated method stub
		List<MPart> parts = modelService.findElements(application, "xyz.veronie.bgg.ui.part.batmain", MPart.class);
		if(parts != null && parts.size() >= 1) {
			MPart fetchPart = parts.get(0);
			partService.showPart(fetchPart, PartState.ACTIVATE);
		}
	}

}